@@ -2,7 +2,13 @@ from __future__ import annotations
 
 import sys
 from typing import (
-    TYPE_CHECKING, AsyncContextManager, Awaitable, Callable, Optional, Tuple, TypeVar,
+    TYPE_CHECKING,
+    AsyncContextManager,
+    Awaitable,
+    Callable,
+    Optional,
+    Tuple,
+    TypeVar,
 )
 
 if sys.version_info >= (3, 8):
@@ -19,7 +25,7 @@ if TYPE_CHECKING:
 # but can also return a value to be returned directly,
 # to skip an unnecessary deserialization (if there was no cache hit).
 # The cache wrap can return either the full result, or only the bytestring part.
-_GF_RET_TV = TypeVar('_GF_RET_TV')
+_GF_RET_TV = TypeVar("_GF_RET_TV")
 TGenerateResult = Tuple[bytes, _GF_RET_TV]
 TCacheResult = Tuple[bytes, Optional[_GF_RET_TV]]
 TGenerateFunc = Callable[[], Awaitable[TGenerateResult]]
@@ -34,5 +40,8 @@ class TClientACM(Protocol):
     If it is requested as exclusive, it should be exclusive for the duration of
     the async context manager, and thrown out if any error happens.
     """
-    def __call__(self, *, master: bool, exclusive: bool) -> AsyncContextManager['Redis']:
+
+    def __call__(
+        self, *, master: bool, exclusive: bool
+    ) -> AsyncContextManager["Redis"]:
         pass
